@@ -18,18 +18,37 @@
 
 #include <inttypes.h>
 
-#define INLINE inline
-
-INLINE static uint8_t
+static inline uint8_t
 clip_to_u8 (int16_t value)
 {
-  return value < 0 ? 0 : (value > 255 ? 255 : value);
+  //return value < 0 ? 0 : (value > 255 ? 255 : value);
+  return ((uint16_t)value) > 256 ? value < 0 ? 0 : 255 : value;
 }
 
+void
+mlib_Init(void)
+{
+}
+
+void
+mlib_VideoIDCTAdd_U8_S16 (uint8_t *curr_block,
+			  int16_t *coeffs, 
+			  int32_t stride) 
+{
+  int x,y;
+  
+  mlib_VideoIDCT8x8_S16_S16(coeffs, coeffs);
+  for (y = 0; y < 8; y++) {
+    for (x = 0; x < 8; x++)
+      curr_block[x] = clip_to_u8(curr_block[x] + *coeffs++);
+    curr_block += stride;
+  }
+}  
+
 void
 mlib_VideoAddBlock_U8_S16 (uint8_t *curr_block,
 			   int16_t *mc_block, 
-                           int32_t stride) 
+			   int32_t stride) 
 {
   int x,y;
 
@@ -43,14 +62,14 @@ mlib_VideoAddBlock_U8_S16 (uint8_t *curr_block,
 void
 mlib_VideoCopyRefAve_U8_U8 (uint8_t *curr_block,
 			    uint8_t *ref_block,
-                            int width, int height,
+			    int width, int height,
 			    int32_t stride)
 {
   int x,y;
   
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = clip_to_u8((curr_block[x] + ref_block[x] + 1)/2);
+      curr_block[x] = (curr_block[x] + ref_block[x] + 1) / 2;
     ref_block += stride;
     curr_block += stride;
   }
@@ -84,9 +103,8 @@ mlib_VideoInterpAveX_U8_U8(uint8_t *curr_block,
 
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = 
-        clip_to_u8((curr_block[x] + 
-		    (ref_block[x] + ref_block[x+1] + 1)/2 + 1) /2);
+      curr_block[x] = (curr_block[x] +
+		       ((ref_block[x] + ref_block[x+1] + 1) / 2) + 1) / 2;
     ref_block += frame_stride;
     curr_block += frame_stride;
   }
@@ -104,9 +122,8 @@ mlib_VideoInterpAveY_U8_U8(uint8_t *curr_block,
 
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = 
-        clip_to_u8((curr_block[x] + 
-		    (ref_block[x] + ref_block_next[x] + 1)/2 + 1)/2);
+      curr_block[x] = (curr_block[x] + 
+		       ((ref_block[x] + ref_block_next[x] + 1) / 2) + 1) / 2;
     curr_block     += frame_stride;
     ref_block      += frame_stride;
     ref_block_next += frame_stride;
@@ -125,10 +142,9 @@ mlib_VideoInterpAveXY_U8_U8(uint8_t *curr_block,
 
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = 
-        clip_to_u8((curr_block[x] + 
-		    (ref_block[x]      + ref_block[x+1] + 
-		     ref_block_next[x] + ref_block_next[x+1] + 2)/4 + 1)/2);
+      curr_block[x] = (curr_block[x] + 
+		       ((ref_block[x] + ref_block_next[x] +
+			 ref_block[x+1] + ref_block_next[x+1] + 2) / 4) + 1) / 2;
     curr_block     += frame_stride;
     ref_block      += frame_stride;
     ref_block_next += frame_stride;
@@ -146,8 +162,7 @@ mlib_VideoInterpX_U8_U8(uint8_t *curr_block,
 
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = 
-        clip_to_u8((ref_block[x] + ref_block[x+1] + 1)/2);
+      curr_block[x] = (ref_block[x] + ref_block[x+1] + 1) / 2;
     ref_block += frame_stride;
     curr_block += frame_stride;
   }
@@ -165,8 +180,7 @@ mlib_VideoInterpY_U8_U8(uint8_t *curr_block,
 
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = 
-        clip_to_u8((ref_block[x] + ref_block_next[x] + 1)/2);
+      curr_block[x] = (ref_block[x] + ref_block_next[x] + 1) / 2;
     curr_block     += frame_stride;
     ref_block      += frame_stride;
     ref_block_next += frame_stride;
@@ -185,9 +199,8 @@ mlib_VideoInterpXY_U8_U8(uint8_t *curr_block,
 
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++)
-      curr_block[x] = 
-        clip_to_u8((ref_block[x]      + ref_block[x+1] + 
-                    ref_block_next[x] + ref_block_next[x+1] + 2)/4);
+      curr_block[x] = (ref_block[x] + ref_block_next[x] +
+		       ref_block[x+1] + ref_block_next[x+1] + 2) / 4;
     curr_block     += frame_stride;
     ref_block      += frame_stride;
     ref_block_next += frame_stride;
