@@ -17,24 +17,31 @@
 #include <stdio.h>
 #include <inttypes.h>
 #include "mmx.h"
-
-extern void IDCT_mmx(int16_t *);
+#include "mmx_mlib.h"
 
 static uint64_t ones = 0x0001000100010001ULL;
 static uint64_t twos = 0x0002000200020002ULL;
 
 //#define MC_MMX_verify
-inline static uint8_t
+
+void
+mlib_Init(void)
+{
+  return;
+}
+
+static inline uint8_t
 clip_to_u8 (int16_t value)
 {
-  return value < 0 ? 0 : (value > 255 ? 255 : value);
+  //return value < 0 ? 0 : (value > 255 ? 255 : value);
+  return ((uint16_t)value) > 256 ? value < 0 ? 0 : 255 : value;
 }
 
 static inline void
 mmx_average_2_U8(uint8_t *dst, uint8_t *src1, uint8_t *src2)
 {
    //
-   // *dst = clip_to_u8((*src1 + *src2 + 1)/2);
+   // *dst = (*src1 + *src2 + 1)/2;
    //
 
    //pxor_r2r(mm0,mm0);         // load 0 into mm0
@@ -67,7 +74,7 @@ static inline void
 mmx_interp_average_2_U8(uint8_t *dst, uint8_t *src1, uint8_t *src2)
 {
    //
-   // *dst = clip_to_u8((*dst + (*src1 + *src2 + 1)/2 + 1)/2);
+   // *dst = (*dst + (*src1 + *src2 + 1)/2 + 1)/2;
    //
 
    //pxor_r2r(mm0,mm0);             // load 0 into mm0
@@ -114,7 +121,7 @@ static inline void
 mmx_average_4_U8(uint8_t *dst, uint8_t *src1, uint8_t *src2, uint8_t *src3, uint8_t *src4)
 {
    //
-   // *dst = clip_to_u8((*src1 + *src2 + *src3 + *src4 + 2)/4);
+   // *dst = (*src1 + *src2 + *src3 + *src4 + 2)/4;
    //
 
    //pxor_r2r(mm0,mm0);                  // load 0 into mm0
@@ -169,7 +176,7 @@ static inline void
 mmx_interp_average_4_U8(uint8_t *dst, uint8_t *src1, uint8_t *src2, uint8_t *src3, uint8_t *src4)
 {
    //
-   // *dst = clip_to_u8((*dst + (*src1 + *src2 + *src3 + *src4 + 2)/4 + 1)/2);
+   // *dst = (*dst + (*src1 + *src2 + *src3 + *src4 + 2)/4 + 1)/2;
    //
 
    //pxor_r2r(mm0,mm0);                  // load 0 into mm0
@@ -236,6 +243,7 @@ mmx_interp_average_4_U8(uint8_t *dst, uint8_t *src1, uint8_t *src2, uint8_t *src
    movq_r2m(mm1,*dst);                 // store result in dst
 }
 
+
 // Add a motion compensated 8x8 block to the current block
 void
 mlib_VideoAddBlock_U8_S16(
@@ -302,7 +310,7 @@ mlib_VideoCopyRefAve_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*curr_block + *ref_block++ + 1)/2);
+         *curr_block++ = (*curr_block + *ref_block++ + 1)/2;
       ref_block += jump;
       curr_block += jump;
    }
@@ -444,12 +452,6 @@ mlib_VideoCopyRef_U8_U8_MxN(
    int x,y;
    const int step = 8;
    const int jump = stride - m;
-#ifdef MC_MMX_verify
-   uint8_t *old_curr_block = curr_block;
-   uint8_t *old_ref_block = ref_block;
-   printf("before\n");
-   print_U8_U8_MxN(n,old_curr_block,old_ref_block,stride);
-#endif
 
    pxor_r2r(mm0,mm0);             // load 0 into mm0
 
@@ -460,18 +462,10 @@ mlib_VideoCopyRef_U8_U8_MxN(
 
          curr_block += step;
          ref_block  += step;
-         //printf("iter n=%d x=%d y=%d curr=%x ref=%x\n",n,x,y,(uint32_t)curr_block,(uint32_t)ref_block);
-         //print_U8_U8_MxN(n,old_curr_block,old_ref_block,stride);
-         //getchar();
       }
       curr_block += jump;
       ref_block  += jump;
    }
-#ifdef MC_MMX_verify
-   printf("after\n");
-   print_U8_U8_MxN(n,old_curr_block,old_ref_block,stride);
-   getchar();
-#endif
 #endif
 }
 
@@ -545,7 +539,7 @@ mlib_VideoInterpAveX_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*curr_block + (*ref_block++ + *(ref_block + 1) + 1)/2 + 1)/2);
+         *curr_block++ = (*curr_block + (*ref_block++ + *(ref_block + 1) + 1)/2 + 1)/2;
       ref_block += jump;
       curr_block += jump;
    }
@@ -645,7 +639,7 @@ mlib_VideoInterpX_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*ref_block++ + *(ref_block + 1) + 1)/2);
+         *curr_block++ = (*ref_block++ + *(ref_block + 1) + 1)/2;
       ref_block += jump;
       curr_block += jump;
    }
@@ -748,7 +742,7 @@ mlib_VideoInterpAveXY_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*curr_block + (*ref_block++ + *(ref_block + 1) + *ref_block_next++ + *(ref_block_next + 1) + 2)/4 + 1)/2);
+         *curr_block++ = (*curr_block + (*ref_block++ + *(ref_block + 1) + *ref_block_next++ + *(ref_block_next + 1) + 2)/4 + 1)/2;
       curr_block     += jump;
       ref_block      += jump;
       ref_block_next += jump;
@@ -853,7 +847,7 @@ mlib_VideoInterpXY_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*ref_block++ + *(ref_block + 1) + *ref_block_next++ + *(ref_block_next + 1) + 2)/4);
+         *curr_block++ = (*ref_block++ + *(ref_block + 1) + *ref_block_next++ + *(ref_block_next + 1) + 2)/4;
       curr_block += jump;
       ref_block += jump;
       ref_block_next += jump;
@@ -960,7 +954,7 @@ mlib_VideoInterpAveY_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*curr_block + (*ref_block++ + *ref_block_next++ + 1)/2 + 1)/2);
+         *curr_block++ = (*curr_block + (*ref_block++ + *ref_block_next++ + 1)/2 + 1)/2;
       curr_block     += jump;
       ref_block      += jump;
       ref_block_next += jump;
@@ -1065,7 +1059,7 @@ mlib_VideoInterpY_U8_U8_MxN(
 
    for (y = 0; y < n; y++) {
       for (x = 0; x < m; x++)
-         *curr_block++ = clip_to_u8((*ref_block++ + *ref_block_next++ + 1)/2);
+         *curr_block++ = (*ref_block++ + *ref_block_next++ + 1)/2;
       curr_block     += jump;
       ref_block      += jump;
       ref_block_next += jump;
@@ -1152,386 +1146,3 @@ mlib_VideoInterpY_U8_U8_8x4(
       frame_stride,
       field_stride);
 }
-
-
-
-
-#define W1 2841 /* 2048*sqrt(2)*cos(1*pi/16) */
-#define W2 2676 /* 2048*sqrt(2)*cos(2*pi/16) */
-#define W3 2408 /* 2048*sqrt(2)*cos(3*pi/16) */
-#define W5 1609 /* 2048*sqrt(2)*cos(5*pi/16) */
-#define W6 1108 /* 2048*sqrt(2)*cos(6*pi/16) */
-#define W7 565  /* 2048*sqrt(2)*cos(7*pi/16) */
-
-
-
-/* row (horizontal) IDCT
- *
- *           7                       pi         1
- * dst[k] = sum c[l] * src[l] * cos( -- * ( k + - ) * l )
- *          l=0                      8          2
- *
- * where: c[0]    = 128
- *        c[1..7] = 128*sqrt(2)
- */
-
-inline static void 
-idct_row(int16_t *blk, int16_t *coeffs)
-{
-  int x0, x1, x2, x3, x4, x5, x6, x7, x8;
-
-  x1 = coeffs[4]<<11; 
-  x2 = coeffs[6]; 
-  x3 = coeffs[2];
-  x4 = coeffs[1]; 
-  x5 = coeffs[7]; 
-  x6 = coeffs[5]; 
-  x7 = coeffs[3];
-
-#if 0
-  /* shortcut */
-  if (!(x1 | x2 | x3 | x4 | x5 | x6 | x7 ))
-  {
-    blk[0]=blk[1]=blk[2]=blk[3]=blk[4]=blk[5]=blk[6]=blk[7]=coeffs[0]<<3;
-    return;
-  }
-#endif
-
-  x0 = (coeffs[0]<<11) + 128; /* for proper rounding in the fourth stage */
-
-  /* first stage */
-  x8 = W7*(x4+x5);
-  x4 = x8 + (W1-W7)*x4;
-  x5 = x8 - (W1+W7)*x5;
-  x8 = W3*(x6+x7);
-  x6 = x8 - (W3-W5)*x6;
-  x7 = x8 - (W3+W5)*x7;
-  
-  /* second stage */
-  x8 = x0 + x1;
-  x0 -= x1;
-  x1 = W6*(x3+x2);
-  x2 = x1 - (W2+W6)*x2;
-  x3 = x1 + (W2-W6)*x3;
-  x1 = x4 + x6;
-  x4 -= x6;
-  x6 = x5 + x7;
-  x5 -= x7;
-  
-  /* third stage */
-  x7 = x8 + x3;
-  x8 -= x3;
-  x3 = x0 + x2;
-  x0 -= x2;
-  x2 = (181*(x4+x5)+128)>>8;
-  x4 = (181*(x4-x5)+128)>>8;
-  
-  /* fourth stage */
-  blk[0] = (x7+x1)>>8;
-  blk[1] = (x3+x2)>>8;
-  blk[2] = (x0+x4)>>8;
-  blk[3] = (x8+x6)>>8;
-  blk[4] = (x8-x6)>>8;
-  blk[5] = (x0-x4)>>8;
-  blk[6] = (x3-x2)>>8;
-  blk[7] = (x7-x1)>>8;
-}
-
-
-/* column (vertical) IDCT
- *
- *             7                         pi         1
- * dst[8*k] = sum c[l] * src[8*l] * cos( -- * ( k + - ) * l )
- *            l=0                        8          2
- *
- * where: c[0]    = 1/1024
- *        c[1..7] = (1/1024)*sqrt(2)
- */
-
-/* FIXME something odd is going on with inlining this 
- * procedure. Things break if it isn't inlined */
-inline static void 
-idct_col(int16_t *blk, int16_t *coeffs)
-{
-  int x0, x1, x2, x3, x4, x5, x6, x7, x8;
-
-  /* shortcut */
-  x1 = (coeffs[8*4]<<8); 
-  x2 = coeffs[8*6]; 
-  x3 = coeffs[8*2];
-  x4 = coeffs[8*1];
-  x5 = coeffs[8*7]; 
-  x6 = coeffs[8*5];
-  x7 = coeffs[8*3];
-
-#if 0
-  if (!(x1  | x2 | x3 | x4 | x5 | x6 | x7 ))
-  {
-    blk[8*0] = blk[8*1] = blk[8*2] = blk[8*3] 
-      = blk[8*4] = blk[8*5] = blk[8*6] = blk[8*7] 
-      = (coeffs[8*0]+32)>>6;
-    return;
-  }
-#endif
-
-  x0 = (coeffs[8*0]<<8) + 8192;
-
-  /* first stage */
-  x8 = W7*(x4+x5) + 4;
-  x4 = (x8+(W1-W7)*x4)>>3;
-  x5 = (x8-(W1+W7)*x5)>>3;
-  x8 = W3*(x6+x7) + 4;
-  x6 = (x8-(W3-W5)*x6)>>3;
-  x7 = (x8-(W3+W5)*x7)>>3;
-  
-  /* second stage */
-  x8 = x0 + x1;
-  x0 -= x1;
-  x1 = W6*(x3+x2) + 4;
-  x2 = (x1-(W2+W6)*x2)>>3;
-  x3 = (x1+(W2-W6)*x3)>>3;
-  x1 = x4 + x6;
-  x4 -= x6;
-  x6 = x5 + x7;
-  x5 -= x7;
-  
-  /* third stage */
-  x7 = x8 + x3;
-  x8 -= x3;
-  x3 = x0 + x2;
-  x0 -= x2;
-  x2 = (181*(x4+x5)+128)>>8;
-  x4 = (181*(x4-x5)+128)>>8;
-  
-  /* fourth stage */
-  blk[8*0] = (x7+x1)>>14;
-  blk[8*1] = (x3+x2)>>14;
-  blk[8*2] = (x0+x4)>>14;
-  blk[8*3] = (x8+x6)>>14;
-  blk[8*4] = (x8-x6)>>14;
-  blk[8*5] = (x0-x4)>>14;
-  blk[8*6] = (x3-x2)>>14;
-  blk[8*7] = (x7-x1)>>14;
-}
-
-inline static void 
-idct_col_u8(uint8_t *blk, int16_t *coeffs, int stride)
-{
-  int x0, x1, x2, x3, x4, x5, x6, x7, x8;
-
-  /* shortcut */
-  x1 = (coeffs[8*4]<<8); 
-  x2 = coeffs[8*6]; 
-  x3 = coeffs[8*2];
-  x4 = coeffs[8*1];
-  x5 = coeffs[8*7]; 
-  x6 = coeffs[8*5];
-  x7 = coeffs[8*3];
-
-#if 0
-  if (!(x1  | x2 | x3 | x4 | x5 | x6 | x7 ))
-  {
-    blk[stride*0] = blk[stride*1] = blk[stride*2] = blk[stride*3]
-      = blk[stride*4] = blk[stride*5] = blk[stride*6] = blk[stride*7]
-      = clip_to_u8(coeffs[stride*0]+32)>>6;
-    return;
-  }
-#endif
-
-  x0 = (coeffs[8*0]<<8) + 8192;
-
-  /* first stage */
-  x8 = W7*(x4+x5) + 4;
-  x4 = (x8+(W1-W7)*x4)>>3;
-  x5 = (x8-(W1+W7)*x5)>>3;
-  x8 = W3*(x6+x7) + 4;
-  x6 = (x8-(W3-W5)*x6)>>3;
-  x7 = (x8-(W3+W5)*x7)>>3;
-  
-  /* second stage */
-  x8 = x0 + x1;
-  x0 -= x1;
-  x1 = W6*(x3+x2) + 4;
-  x2 = (x1-(W2+W6)*x2)>>3;
-  x3 = (x1+(W2-W6)*x3)>>3;
-  x1 = x4 + x6;
-  x4 -= x6;
-  x6 = x5 + x7;
-  x5 -= x7;
-  
-  /* third stage */
-  x7 = x8 + x3;
-  x8 -= x3;
-  x3 = x0 + x2;
-  x0 -= x2;
-  x2 = (181*(x4+x5)+128)>>8;
-  x4 = (181*(x4-x5)+128)>>8;
-  
-  /* fourth stage */
-  blk[stride*0] = clip_to_u8((x7+x1)>>14);
-  blk[stride*1] = clip_to_u8((x3+x2)>>14);
-  blk[stride*2] = clip_to_u8((x0+x4)>>14);
-  blk[stride*3] = clip_to_u8((x8+x6)>>14);
-  blk[stride*4] = clip_to_u8((x8-x6)>>14);
-  blk[stride*5] = clip_to_u8((x0-x4)>>14);
-  blk[stride*6] = clip_to_u8((x3-x2)>>14);
-  blk[stride*7] = clip_to_u8((x7-x1)>>14);
-}
-
-
-
-void
-mlib_VideoIDCT8x8_S16_S16(int16_t *block,
-			  int16_t *coeffs)
-{
-#ifndef HAVE_MMX
-  int i;
-  
-  for (i=0; i<8; i++)
-    idct_row(block + 8*i, coeffs + 8*i);
-  for (i=0; i<8; i++)
-    idct_col(block + i, block + i);
-#else
-  
-  movq_m2r(*coeffs, mm0);
-  movq_m2r(*(coeffs+4), mm1);
-  psllw_i2r(4, mm0);
-  movq_m2r(*(coeffs+8), mm2);
-  psllw_i2r(4, mm1);
-  movq_m2r(*(coeffs+12), mm3);
-  psllw_i2r(4, mm2);
-  movq_m2r(*(coeffs+16), mm4);
-  psllw_i2r(4, mm3);
-  movq_m2r(*(coeffs+20), mm5);
-  psllw_i2r(4, mm4);
-  movq_m2r(*(coeffs+24), mm6);
-  psllw_i2r(4, mm5);
-  movq_m2r(*(coeffs+28), mm7);
-  psllw_i2r(4, mm6);
-  psllw_i2r(4, mm7);
-  movq_r2m(mm0, *block);
-  movq_r2m(mm1, *(block+4));
-  movq_r2m(mm2, *(block+8));
-  movq_r2m(mm3, *(block+12));
-  movq_r2m(mm4, *(block+16));
-  movq_r2m(mm5, *(block+20));
-  movq_r2m(mm6, *(block+24));
-  movq_r2m(mm7, *(block+28));
-  movq_m2r(*(coeffs+32), mm0);
-  movq_m2r(*(coeffs+36), mm1);
-  psllw_i2r(4, mm0);
-  movq_m2r(*(coeffs+40), mm2);
-  psllw_i2r(4, mm1);
-  movq_m2r(*(coeffs+44), mm3);
-  psllw_i2r(4, mm2);
-  movq_m2r(*(coeffs+48), mm4);
-  psllw_i2r(4, mm3);
-  movq_m2r(*(coeffs+52), mm5);
-  psllw_i2r(4, mm4);
-  movq_m2r(*(coeffs+56), mm6);
-  psllw_i2r(4, mm5);
-  movq_m2r(*(coeffs+60), mm7);
-  psllw_i2r(4, mm6);
-  psllw_i2r(4, mm7);
-  movq_r2m(mm0, *(block+32));
-  movq_r2m(mm1, *(block+36));
-  movq_r2m(mm2, *(block+40));
-  movq_r2m(mm3, *(block+44));
-  movq_r2m(mm4, *(block+48));
-  movq_r2m(mm5, *(block+52));
-  movq_r2m(mm6, *(block+56));
-  movq_r2m(mm7, *(block+60));
-	
-  IDCT_mmx( block );
-#endif
-}
-
-
-void
-mlib_VideoIDCT8x8_U8_S16(uint8_t *block,
-			 int16_t *coeffs,
-			 int32_t stride)
-{
-#ifndef HAVE_MMX
-  int16_t temp[64];
-  int i;
-  
-  for (i=0; i<8; i++)
-    idct_row(temp + 8*i, coeffs + 8*i);
-  for (i=0; i<8; i++)
-    idct_col_u8(block + i, temp + i, stride);
-#else
-  int x,y;
-  const int n = 8;
-  const int jump = stride - n;
-
-  movq_m2r(*coeffs, mm0);
-  movq_m2r(*(coeffs+4), mm1);
-  psllw_i2r(4, mm0);
-  movq_m2r(*(coeffs+8), mm2);
-  psllw_i2r(4, mm1);
-  movq_m2r(*(coeffs+12), mm3);
-  psllw_i2r(4, mm2);
-  movq_m2r(*(coeffs+16), mm4);
-  psllw_i2r(4, mm3);
-  movq_m2r(*(coeffs+20), mm5);
-  psllw_i2r(4, mm4);
-  movq_m2r(*(coeffs+24), mm6);
-  psllw_i2r(4, mm5);
-  movq_m2r(*(coeffs+28), mm7);
-  psllw_i2r(4, mm6);
-  psllw_i2r(4, mm7);
-  movq_r2m(mm0, *coeffs);
-  movq_r2m(mm1, *(coeffs+4));
-  movq_r2m(mm2, *(coeffs+8));
-  movq_r2m(mm3, *(coeffs+12));
-  movq_r2m(mm4, *(coeffs+16));
-  movq_r2m(mm5, *(coeffs+20));
-  movq_r2m(mm6, *(coeffs+24));
-  movq_r2m(mm7, *(coeffs+28));
-  movq_m2r(*(coeffs+32), mm0);
-  movq_m2r(*(coeffs+36), mm1);
-  psllw_i2r(4, mm0);
-  movq_m2r(*(coeffs+40), mm2);
-  psllw_i2r(4, mm1);
-  movq_m2r(*(coeffs+44), mm3);
-  psllw_i2r(4, mm2);
-  movq_m2r(*(coeffs+48), mm4);
-  psllw_i2r(4, mm3);
-  movq_m2r(*(coeffs+52), mm5);
-  psllw_i2r(4, mm4);
-  movq_m2r(*(coeffs+56), mm6);
-  psllw_i2r(4, mm5);
-  movq_m2r(*(coeffs+60), mm7);
-  psllw_i2r(4, mm6);
-  psllw_i2r(4, mm7);
-  movq_r2m(mm0, *(coeffs+32));
-  movq_r2m(mm1, *(coeffs+36));
-  movq_r2m(mm2, *(coeffs+40));
-  movq_r2m(mm3, *(coeffs+44));
-  movq_r2m(mm4, *(coeffs+48));
-  movq_r2m(mm5, *(coeffs+52));
-  movq_r2m(mm6, *(coeffs+56));
-  movq_r2m(mm7, *(coeffs+60));
-
-  IDCT_mmx( coeffs );
-  
-  pxor_r2r(mm0, mm0);             // load 0 into mm0
-  
-  for (y = 0; y < n; y++) {
-    for (x = 0; x < n/8; x++) {
-      movq_m2r(*coeffs, mm1);         // load 4 mc words (mc0)
-      movq_m2r(*(coeffs+4), mm2);     // load next 4 mc words (mc1)
-      
-      packuswb_r2r(mm2, mm1);         // pack (w/ saturation)
-      movq_r2m(mm1, *block);          // store result in curr
-      
-      block  += stride;
-      coeffs += n;
-    }
-  }
-#endif
-}  
-
-
-
