@@ -37,9 +37,11 @@
 #else
 #ifdef HAVE_MMX
 #include "mmx.h"
-#endif
+#include "mmx_mlib.h"
+#else
 #include "c_mlib.h"
 #endif
+#endif
 
 #include "common.h"
 
@@ -164,19 +166,6 @@ void reset_vectors()
 
 
 
-
-static inline
-void inverse_quantisation_final(int sum)
-{
-  if((sum & 1) == 0) {
-    if((mb.QFS[63]&1) != 0) {
-      mb.QFS[63] = mb.QFS[63]-1;
-    } else {
-      mb.QFS[63] = mb.QFS[63]+1;
-    }
-  }
-}
-
 #if NEW_TABLES
 /* 6.2.6 Block */
 static
@@ -192,10 +181,10 @@ void block_intra(unsigned int i)
   
   //cpc_count_usr_events(1);
   
-  { // Reset all coefficients to 0.
+  { /* Reset all coefficients to 0. */
     int m;
     for(m=0; m<16; m++)
-      memset( ((uint64_t *)mb.QFS) + m, 0, sizeof(uint64_t) );
+      *(((uint64_t *)mb.QFS) + m) = 0;
   }
     
     
@@ -252,7 +241,7 @@ void block_intra(unsigned int i)
 	} 
 #endif
 	mb.QFS[0] = f;
-	inverse_quantisation_sum = f;
+	inverse_quantisation_sum = f + 1;
       }
       n = 1;
     }
@@ -318,8 +307,8 @@ void block_intra(unsigned int i)
   }
   
   //cpc_count_usr_events(0);
-  
-  inverse_quantisation_final(inverse_quantisation_sum);
+
+  mb.QFS[63] ^= inverse_quantisation_sum & 1;
   
   DPRINTF(4, "nr of coeffs: %d\n", n);
 }
@@ -335,16 +324,16 @@ void block_non_intra(unsigned int b)
   unsigned int bits;
   
   unsigned int n = 0;
-  int inverse_quantisation_sum = 0;
+  int inverse_quantisation_sum = 1;
   
   DPRINTF(3, "pattern_code(%d) set\n", b);
   
   //cpc_count_usr_events(1);
 
-  { // Reset all coefficients to 0.
+  { /* Reset all coefficients to 0. */
     int m;
-    for(m = 0; m < 16; m++)
-      memset((uint64_t *)mb.QFS + m, 0, sizeof(uint64_t));
+    for(m=0; m<16; m++)
+      *(((uint64_t *)mb.QFS) + m) = 0;
   }
   
   // We need the first and the following 8 bits
@@ -403,7 +392,7 @@ void block_non_intra(unsigned int b)
   
   //cpc_count_usr_events(0);
   
-  inverse_quantisation_final(inverse_quantisation_sum);
+  mb.QFS[63] ^= inverse_quantisation_sum & 1;
   
   DPRINTF(4, "nr of coeffs: %d\n", n);
 }
@@ -427,13 +416,12 @@ void block_intra(unsigned int i)
   
   //cpc_count_usr_events(1);
   
-  { // Reset all coefficients to 0.
+  { /* Reset all coefficients to 0. */
     int m;
     for(m=0; m<16; m++)
-      memset( ((uint64_t *)mb.QFS) + m, 0, sizeof(uint64_t) );
+      *(((uint64_t *)mb.QFS) + m) = 0;
   }
     
-    
   /* DC - component */
   {
     unsigned int dct_dc_size;
@@ -487,7 +475,7 @@ void block_intra(unsigned int i)
 	} 
 #endif
 	mb.QFS[0] = f;
-	inverse_quantisation_sum = f;
+	inverse_quantisation_sum = f + 1;
       }
       n = 1;
     }
@@ -635,7 +623,7 @@ void block_intra(unsigned int i)
 
   //cpc_count_usr_events(0);
   
-  inverse_quantisation_final(inverse_quantisation_sum);
+  mb.QFS[63] ^= inverse_quantisation_sum & 1;
   
   // Clean-up
   bits_left = left;
@@ -652,7 +640,7 @@ static
 void block_non_intra(unsigned int b)
 {
   unsigned int n = 0;
-  int inverse_quantisation_sum = 0;
+  int inverse_quantisation_sum = 1;
   
   /* Make a local 'getbits' implementation. */
   unsigned int left = bits_left;
@@ -664,11 +652,10 @@ void block_non_intra(unsigned int b)
   
   //cpc_count_usr_events(1);    
  
-  // Reset all coefficients to 0.
-  {
+  { /* Reset all coefficients to 0. */
     int m;
-    for(m = 0; m < 16; m++)
-      memset((uint64_t *)mb.QFS + m, 0, sizeof(uint64_t));
+    for(m=0; m<16; m++)
+      *(((uint64_t *)mb.QFS) + m) = 0;
   }
   
   while(1) {
@@ -803,7 +790,7 @@ void block_non_intra(unsigned int b)
   
   //cpc_count_usr_events(0);
 
-  inverse_quantisation_final(inverse_quantisation_sum);
+  mb.QFS[63] ^= inverse_quantisation_sum & 1;
   
   // Clean-up
   bits_left = left;
@@ -1174,27 +1161,45 @@ void mpeg2_slice(void)
 	/* In a P-picture when a macroblock is skipped */
 	reset_PMV();
 	reset_vectors();
-	/* mlib is broken!!! 
-	{
-	   int x = (seq.mb_column-macroblock_address_increment);
-	   int y = seq.mb_row;
-	   int offs_y = x * 16 + y * 16 * seq.mb_width * 16;
-	   int offs_uv = x * 8 + y *  8 * (seq.mb_width * 16)/2;
-	   mlib_VideoCopyRef_U8_U8(&dst_image->y[offs_y],
-				   &fwd_ref_image->y[offs_y], 
-				   macroblock_address_increment*16, 
-				   16, (seq.mb_width * 16));
-	   mlib_VideoCopyRef_U8_U8(&dst_image->u[offs_uv],
-				   &fwd_ref_image->u[offs_uv],
-				   macroblock_address_increment*8,
-				   8, (seq.mb_width * 16)/2);
-	   mlib_VideoCopyRef_U8_U8(&dst_image->v[offs_uv],
-				   &fwd_ref_image->v[offs_uv],
-				   macroblock_address_increment*8,
-				   8, (seq.mb_width * 16)/2);
-        }
-	*/
 	if(pic.coding_ext.picture_structure == PIC_STRUCT_FRAME_PICTURE) {
+	/* mlib is broken!!! 
+	   {
+	     int x = (seq.mb_column-macroblock_address_increment);
+	     int y = seq.mb_row;
+	     int offs_y = x * 16 + y * 16 * seq.mb_width * 16;
+	     int offs_uv = x * 8 + y *  8 * (seq.mb_width * 16)/2;
+	     mlib_VideoCopyRef_U8_U8(&dst_image->y[offs_y],
+				     &fwd_ref_image->y[offs_y], 
+				     macroblock_address_increment*16, 
+				     16, (seq.mb_width * 16));
+	     mlib_VideoCopyRef_U8_U8(&dst_image->u[offs_uv],
+				     &fwd_ref_image->u[offs_uv],
+				     macroblock_address_increment*8,
+				     8, (seq.mb_width * 16)/2);
+	     mlib_VideoCopyRef_U8_U8(&dst_image->v[offs_uv],
+				     &fwd_ref_image->v[offs_uv],
+				     macroblock_address_increment*8,
+				     8, (seq.mb_width * 16)/2);
+          }
+	*/
+#if HAVE_ALTIVEC
+	  const int x = seq.mb_column - macroblock_address_increment;
+	  const int y = seq.mb_row;
+	  const int offs_y = x * 16 + y * 16 * seq.mb_width * 16;
+	  const int offs_uv = x * 8 + y *  8 * seq.mb_width *  8;
+	  mlib_VideoCopyRef_U8_U8_16x16_multiple(&dst_image->y[offs_y],
+						 &fwd_ref_image->y[offs_y],
+						 seq.mb_width * 16,
+						 macroblock_address_increment);
+	  mlib_VideoCopyRef_U8_U8_8x8_multiple(&dst_image->u[offs_uv],
+					       &fwd_ref_image->u[offs_uv],
+					       seq.mb_width * 8,
+					       macroblock_address_increment);
+	  mlib_VideoCopyRef_U8_U8_8x8_multiple(&dst_image->v[offs_uv],
+					       &fwd_ref_image->v[offs_uv],
+					       seq.mb_width * 8,
+					       macroblock_address_increment);
+#else
 	  unsigned int x, y;
 	  /* 7.6.6.2 P frame picture */
 	  x = (seq.mb_column-macroblock_address_increment)*16;
@@ -1214,6 +1219,7 @@ void mpeg2_slice(void)
 		   &fwd_ref_image->v[y*(seq.mb_width*16)/2+x], 
 		   macroblock_address_increment*8);
 	  }
+#endif
 	} else {
 	  // Optimize this case too?  Maybe move all this to video_motion
 	  unsigned int old_col = seq.mb_column;
@@ -1456,7 +1462,6 @@ void mpeg2_slice(void)
 	    if(pic.coding_ext.picture_structure == PIC_STRUCT_BOTTOM_FIELD)
 	      dst += stride/2;
 	  }
-	  
 	  mlib_VideoIDCT8x8_U8_S16(dst, (int16_t *)mb.QFS, stride);
 	}
       }
@@ -1473,8 +1478,8 @@ void mpeg2_slice(void)
 	  if(mb.cbp & (1<<(5-i))) {
 	    DPRINTF(4, "cbpindex: %d set\n", i);
 	    block_non_intra(i);
-	    mlib_VideoIDCT8x8_S16_S16((int16_t *)mb.QFS, (int16_t *)mb.QFS);
-	    motion_comp_add_coeff(i);
+	    //mlib_VideoIDCT8x8_S16_S16((int16_t *)mb.QFS, (int16_t *)mb.QFS);
+	    motion_comp_add_coeff(i); // IDCT and add done in here..
 	  }
 	}
 #if 0
