@@ -33,6 +33,7 @@
 #include <mlib_sys.h>
 #include <mlib_video.h>
 #include <mlib_algebra.h>
+#define mlib_Init() (void)0
 #else
 #ifdef HAVE_MMX
 #include "mmx.h"
@@ -138,7 +139,7 @@ slice_t slice_data;
 #if PRAGMA_ALIGN
 #pragma align 32 (mb)
 #endif
-macroblock_t mb;
+macroblock_t ATTRIBUTE_ALIGNED(32) mb;
 
 
 yuv_image_t *image_bufs;
@@ -219,50 +220,30 @@ void fprintbits(FILE *fp, unsigned int bits, uint32_t value)
 
 
 int get_vlc(const vlc_table_t *table, char *func) {
-  int pos=0;
-  int numberofbits=1;
+  int numberofbits;
   int vlc;
-#if 1
-  while(table[pos].numberofbits != VLC_FAIL) {
-    vlc = nextbits(numberofbits);
-    while(table[pos].numberofbits == numberofbits) {
-      if(table[pos].vlc == vlc) {
-	DPRINTF(3, "get_vlc(%s): len: %d, vlc: %d, val: %d\n",
-		func, numberofbits, table[pos].vlc, table[pos].value);
-	dropbits(numberofbits);
-	return (table[pos].value);
-      }
-      pos++;
-    }
-    numberofbits++;
-  }
-#else
-  int found = 0;
-  numberofbits=0;
+  
+  vlc = nextbits(16);
   while(1) {
-    if(table[pos].numberofbits != numberofbits) {
-      numberofbits = table[pos].numberofbits;
-      if(numberofbits == VLC_FAIL)
-	break;
-      vlc = nextbits(numberofbits);
-    }
-    if(table[pos].vlc == vlc) {
-      found = 1;
-      break;
-    }
-    pos++;
-  }
-  if(found) {
-    dropbits(numberofbits);
-    return (table[pos].value);
-  }
+    numberofbits = table->numberofbits;
+    if(numberofbits == VLC_FAIL) {
+#if 0
+      fprintf(stderr, "*** get_vlc(vlc_table *table, \"%s\"): no matching " 
+	      "bitstream found.\nnext 32 bits: %08x, ", func, nextbits(32));
+      fprintbits(stderr, 32, nextbits(32));
+      fprintf(stderr, "\n");
+      //exit_program(1);
 #endif
-  fprintf(stderr, "*** get_vlc(vlc_table *table, \"%s\"): no matching " 
-	  "bitstream found.\nnext 32 bits: %08x, ", func, nextbits(32));
-  fprintbits(stderr, 32, nextbits(32));
-  fprintf(stderr, "\n");
-  //exit_program(1);
-  return VLC_FAIL;
+      return VLC_FAIL;
+    }
+    if(table->vlc == (vlc>>(16-numberofbits)))
+      break;
+    table++;
+  }
+  DPRINTF(3, "get_vlc(%s): len: %d, vlc: %d, val: %d\n",
+	  func, numberofbits, table->vlc, table->value);
+  dropbits(numberofbits);
+  return table->value;
 }
 
 
@@ -323,7 +304,7 @@ int main(int argc, char **argv)
   }
   
   init_program();
-  
+  mlib_Init();
   //init_out_q(nr_of_buffers);
   
   if(msgqid == -1) {
@@ -588,20 +569,10 @@ void sequence_header(void)
     int n;
     intra_inverse_quantiser_matrix_changed = 1;
     
-    for(n = 0; n < 64; n++) {
-      seq.header.intra_quantiser_matrix[n] 
-	= GETBITS(8, "intra_quantiser_matrix[n]");
-    }
-    
     /* 7.3.1 Inverse scan for matrix download */
-    {
-      int v, u;
-      for (v=0; v<8; v++) {
-	for (u=0; u<8; u++) {
-	  seq.header.intra_inverse_quantiser_matrix[v*8+u] =
-	    seq.header.intra_quantiser_matrix[scan[0][v][u]];
-	}
-      }
+    for(n = 0; n < 64; n++) {
+      seq.header.intra_inverse_quantiser_matrix[inverse_scan[0][n]] =
+	GETBITS(8, "intra_quantiser_matrix[n]");
     }
     
   } else {
@@ -615,20 +586,10 @@ void sequence_header(void)
     int n;
     non_intra_inverse_quantiser_matrix_changed = 1;
     
-    for(n = 0; n < 64; n++) {
-      seq.header.non_intra_quantiser_matrix[n] 
-	= GETBITS(8, "non_intra_quantiser_matrix[n]");
-    }
-   
     /** 7.3.1 Inverse scan for matrix download */
-    {
-      int v, u;
-      for (v=0; v<8; v++) {
-	for (u=0; u<8; u++) {
-	  seq.header.non_intra_inverse_quantiser_matrix[v*8+u] =
-	    seq.header.non_intra_quantiser_matrix[scan[0][v][u]];
-	}
-      }
+    for(n = 0; n < 64; n++) {
+      seq.header.non_intra_inverse_quantiser_matrix[inverse_scan[0][n]] =
+	GETBITS(8, "non_intra_quantiser_matrix[n]");
     }
     
   } else {
@@ -2205,42 +2166,24 @@ void quant_matrix_extension()
     int n;
     intra_inverse_quantiser_matrix_changed = 1;
 
+    /* 7.3.1 Inverse scan for matrix download */
     for(n = 0; n < 64; n++) {
-      seq.header.intra_quantiser_matrix[n] 
-	= GETBITS(8, "intra_quantiser_matrix[n]");
+      seq.header.intra_inverse_quantiser_matrix[inverse_scan[0][n]] =
+	GETBITS(8, "intra_quantiser_matrix[n]");
     }
     
-    /* 7.3.1 Inverse scan for matrix download */
-    {
-      int v, u;
-      for (v=0; v<8; v++) {
-	for (u=0; u<8; u++) {
-	  seq.header.intra_inverse_quantiser_matrix[v*8+u] =
-	    seq.header.intra_quantiser_matrix[scan[0][v][u]];
-	}
-      }
-    }
   }
     
   if(GETBITS(1, "load_non_intra_quantiser_matrix")) {
     int n;
     non_intra_inverse_quantiser_matrix_changed = 1;
     
+    /*  7.3.1 Inverse scan for matrix download */
     for(n = 0; n < 64; n++) {
-      seq.header.non_intra_quantiser_matrix[n] 
-	= GETBITS(8, "non_intra_quantiser_matrix[n]");
+      seq.header.non_intra_inverse_quantiser_matrix[inverse_scan[0][n]] =
+	GETBITS(8, "non_intra_quantiser_matrix[n]");
     }
    
-    /*  7.3.1 Inverse scan for matrix download */
-    {
-      int v, u;
-      for (v=0; v<8; v++) {
-	for (u=0; u<8; u++) {
-	  seq.header.non_intra_inverse_quantiser_matrix[v*8+u] =
-	    seq.header.non_intra_quantiser_matrix[scan[0][v][u]];
-	}
-      }
-    }
   }
   GETBITS(1, "load_chroma_intra_quantiser_matrix (always 0 in 4:2:0)");
   GETBITS(1, "load_chroma_non_intra_quantiser_matrix (always 0 in 4:2:0)");
